#include <avr/interrupt.h>
#include <util/delay.h>

//f, a, c, g si f din octava urmatoare
#define f  349
#define a  440
#define c  523
#define g2 698
#define f2 783
#define p    0

//note duration, pause duration
#define nd  7000
#define pd  17

//notele cantate, in hertzi
uint16_t notes[] = {
                   p,f,p,f,p,f,p,f,p,f,p,f,p,f,p,f,p,f,p,f,p,f,p,    p,p,    a,p,                     //randul 1
                   a,p,a,p,a,p,a,p,    p,p,p,p,    a,p,a,p,a,p,a,p,    p,p,p,    a,p,                 //randul 2
                   c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,c,p,                   //randul 3
                   f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p,f2,p    //randul 4
};

//duratiile notelor, in something that kinda works
uint16_t durations[] = {
                       pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,nd,pd,    nd,nd,    nd,    nd,
                       nd,pd,nd,pd,nd,pd,nd,pd,    nd,nd,nd,nd,    nd,pd,nd,pd,nd,pd,nd,pd,nd,nd,nd,    nd,pd,
                       nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,
                       nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,    nd,pd,nd,pd,nd,pd,nd,pd,
};

//numarul de note care vor fi cantate
uint8_t num_notes = sizeof(notes) / sizeof(notes[0]);

void timer1_init()
{
	//timer1 este configurat cu modul FastPWM, top ICR1 si prescaler 1024
	TCCR1A |= (2 << WGM10);
	TCCR1B |= (5 << CS10) | (3 << WGM12);

	TCCR1A |= (2 << COM1B0) | (2 << COM1A0);

	//modul curent numara pana la ICR1
	ICR1 = 60000;
	//duty cycle-ul este setat initial la 25%
	OCR1B = ICR1 / 4;
	//duty cycle-ul initial pentru speaker (OC1A) este 50%
	OCR1A = ICR1 / 2;
}

int main(void)
{
	//speaker-ul si led-urile sunt puse ca iesire
	DDRD |= (1 << PD5);
	DDRA |= 0xFF;
	DDRC |= 0xFF;
	//tranzistorii care controleaza fiecare plan
	DDRB |= 0xFF;

	//initializam PWM-ul
	timer1_init();

	uint8_t note_remaining = 1;
	int8_t note_position = -1;

	//initial, toate ledurile sunt aprinse
	PORTA |= 0xFF;
	PORTC |= 0xFF;

	//initial, doar primul plan este aprins
	PORTB = 1;

	int iter = 0;
	int led_index = 0;
	int level_index = 1;
	while(1)
	{
		//muzica
		if(note_remaining == 0)
		{
			note_position = (note_position + 1) % num_notes;
			note_remaining = durations[note_position];
			ICR1 = 16000000 / 1024 / notes[note_position];

			//match cand counter-ul este la ICR1/8 <=> duty cycle
			OCR1A = ICR1 / 8;
			//initial, counter-ul e 0
			TCNT1 = 0;
		}
		note_remaining--;

		//leduri
		if(iter%100 == 0){
			//daca am ajuns la ultimul led, trec la nivelul urmator
			if(led_index == 16){
				led_index = 0;
				PORTB = 1 << level_index;
				level_index++;
				if(level_index == 4)
					level_index = 0;
			}
			if(led_index<8){
				switch(led_index){
					case 0: PORTC ^= 1 << 0; break;
					case 1: PORTC ^= 1 << 1; break;
					case 2: PORTC ^= 1 << 2; break;
					case 3: PORTC ^= 1 << 3; break;
					case 4: PORTC ^= 1 << 7; break;
					case 5: PORTC ^= 1 << 6; break;
					case 6: PORTC ^= 1 << 5; break;
					case 7: PORTC ^= 1 << 4; break;
				}
			} else {
				switch(led_index){
					case 8: PORTA ^= 1 << 7; break;
					case 9: PORTA ^= 1 << 6; break;
					case 10: PORTA ^= 1 << 5; break;
					case 11: PORTA ^= 1 << 4; break;
					case 12: PORTA ^= 1 << 0; break;
					case 13: PORTA ^= 1 << 1; break;
					case 14: PORTA ^= 1 << 2; break;
					case 15: PORTA ^= 1 << 3; break;
				}
			}
			led_index++;
		}
		iter++;
		_delay_ms(25);
	}

	return 0;
}

